function [outmsg rotation_num] = RotationCipher(InMsg, Dictionary)
% This function takes in a msg that has been put through a rotation cipher
% and a cell array of words and returns the most likely output message as
% well as the number for the rotation (ie 1 would make all of the a's into
% b's).

% Make it all lower case
InMsg = lower(InMsg);

% Create a bit mask for where the letters are in the original image
letters = InMsg >= 'a' & InMsg <= 'z';

% Initialize some variables
bestprob = 0;
bestrotation = -1;
bestmessage = '';

% For each of the possible rotations (including no rotation)
for i = 0:25
    % Add i to all of the letters
    RotatedMsg = InMsg;
    RotatedMsg(letters) = char(RotatedMsg(letters) + i);
    % Gets a mask for all of the out of range letters
    outofrange = letters & RotatedMsg > 'z';
    % Subtracts 26 from the letters that got pushed out of range
    RotatedMsg(outofrange) = char(RotatedMsg(outofrange) - 26);
    % Gets a cell array of 
    RotatedWords = GetWords(RotatedMsg);
    % Finds the probability that these are all correct words
    probability = ProbabilityWords(RotatedWords, Dictionary);
    % If it's the best probability so far, assign it as the best
    if (probability > bestprob)
        bestprob = probability;
        bestrotation = i;
        bestmessage = RotatedMsg;
    end
end

% Return the best message and the rotation number
outmsg = bestmessage;
rotation_num = bestrotation;

end