function DecodedMsg = ShreaderCipher(InMsg, charDict, wordDict)
% This function takes in a "Shreader Cipher" where the original message has
% been "cut up" into smaller pieces that have to be pieced together. The
% function also takes in a cell array containing an alphabetical list of
% four character sequences taken from some source text along with the
% number of times that sequence occurs.

numbercolumns = length(InMsg);
possiblechoices = [];
    
% Interate through all of the choices for the far left column
for startcol = 1:numbercolumns
    availablecols = 1:numbercolumns;
    correctorder = startcol;
    availablecols(availablecols==startcol) = [];
    % Repeat until all of the columns have been assigned
    while (~isempty(availablecols))
        bestchoice = -1;
        bestprob = 0;
        % Try each available column to see how well it matches the last one
        for col = availablecols
            curprob = ProbColsMatch(InMsg{correctorder(end)},InMsg{col},charDict);
            if (curprob > bestprob)
                bestchoice = col;
                bestprob = curprob;
            end
        end
        correctorder = [correctorder bestchoice];
        availablecols(availablecols==bestchoice) = [];
    end
    possiblechoices = [possiblechoices; correctorder];
end

bestprobability = 0;
bestarrangement = [];
% For all of the possible choices
for i=1:numbercolumns
    % Get the text for each choice
    curchoice = possiblechoices(i,:);
    curarrangement = cell2mat(InMsg(curchoice));
    % Parse out all of the words
    Words = [];
    for row = 1:size(curarrangement,1)
        Words = [Words GetWords(curarrangement(row,:))];
    end
    probability = ProbabilityWords(Words, wordDict);
    if (probability > bestprobability)
        bestarrangement = curarrangement;
        bestprobability = probability;
    end
end

DecodedMsg = bestarrangement;

end